# Generated by Django 2.0.6 on 2018-08-04 14:31

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_telethon_authorization', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='telegramauthorization',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterUniqueTogether(
            name='telegramauthorization',
            unique_together={('user', 'phone')},
        ),
    ]
