# Generated by Django 2.0.6 on 2018-08-04 14:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_telethon_authorization', '0002_auto_20180804_1431'),
    ]

    operations = [
        migrations.AlterField(
            model_name='telegramauthorization',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterUniqueTogether(
            name='telegramauthorization',
            unique_together=set(),
        ),
    ]
